
## ----setup, include=FALSE------------------------------------------------
library(knitr)
opts_chunk$set(out.extra='style="display:block; margin: auto"'
    , fig.align="left"
    , fig.width=4.6*1.2, fig.height=3.2*1.2, dev.args=list(pointsize=10)
    )
knit_hooks$set(spar = function(before, options, envir) {
    if (before){
        par( las=1 )                   #also y axis labels horizontal
        par(mar=c(2.0,3.5,0,0)+0.3 )  #margins
        par(tck=0.02 )                          #axe-tick length inside plots             
        par(mgp=c(1.1,0.2,0) )  #positioning of axis title, axis labels, axis
     }
})


## ----, result='hide'-----------------------------------------------------
library(twNlme)


## ----, spar=TRUE---------------------------------------------------------
data(Wutzler08BeechStem)
head(Wutzler08BeechStem)
ds <- Wutzler08BeechStem[order(Wutzler08BeechStem$dbh),]
plot( stem~dbh, col=author, data=ds, xlab="dbh [cm]", ylab="")
mtext( "stem biomass [kg]", 2, 2.8, las=0)
legend( "topleft", inset=c(0.01,0.01), levels(ds$author), col=1:13, pch=1 )


## ----, spar=TRUE---------------------------------------------------------
plot( log(stem)~log(dbh), col=author, data=subset(ds, dbh >= 7) )


## ----, spar=TRUE---------------------------------------------------------
lm1 <- lm( log(stem)~log(dbh), data=ds )
.start <- structure( c( exp(coef(lm1)[1]), coef(lm1)[2] ), names=c("b0","b1")) 

gm1 <- gnls( stem ~ b0 * dbh^b1 
    , data=ds 
    , params=c(b0+b1~1)
    , start=.start   
    , weights=varPower(form = ~fitted(.))
)
gm1

plot( stem~dbh, data=ds, xlab="dbh [cm]", ylab="")
mtext( "stem biomass [kg]", 2, 2.5, las=0)
lines( fitted(gm1) ~ dbh, data=ds, col="black", lwd=2 )

sdEps <- sqrt( varResidPower(gm1, pred=fitted(gm1)) )
gm1a <- attachVarPrep(gm1, fVarResidual=varResidPower)
sd <- varPredictNlmeGnls(gm1a, newdata=ds)[,"sdInd"] 
lines( fitted(gm1)+sd ~ dbh, data=ds, col="grey" )
lines( fitted(gm1)-sd ~ dbh, data=ds, col="grey" )


## ----, spar=TRUE---------------------------------------------------------
apply( varPredictNlmeGnls(gm1a, newdata=ds), 2, median )


## ----predictNoGroup, spar=TRUE, cache=TRUE-------------------------------
n <- 1000      # sum over n trees
set.seed(0815)
dsNew <- data.frame(dbh=rnorm(n, mean=30, sd=2), author=ds$author[1])
yNew <- predict(gm1a, newdata=dsNew )
varNew <- varPredictNlmeGnls(gm1a, newdata=dsNew)
median( varNew[,"sdInd"]/yNew )       # on average each tree biomass with 33% uncertainty
yAgg <- sum(yNew)
# completely independent:
cvYAggRes <- sqrt( sum(varNew[,"varResid"]) ) /yAgg       # only 1% uncertainty ?
# indpendent but account for uncertainty in model coefficients
varSumComp <- varSumPredictNlmeGnls(gm1a, newdata=dsNew, retComponents = TRUE)
cvYAggInd <- varSumComp["sdPred"]/yAgg       # 3.1%
# dependent errors: add standard deviation
cvYAggDep <- sum( sqrt(varNew[,"varResid"]) ) /yAgg       # 33%: no decrease with increasing n
structure( c( cvYAggRes, cvYAggInd, cvYAggDep )*100, names=c("Residuals","Independent","Correlated"))
varSumComp/(n*10)


## ----nlmeFit, spar=TRUE, cache=TRUE--------------------------------------
mm1 <- nlme( stem ~ b0 * dbh^b1 
        , data=ds 
        , fixed=c(b0+b1~1)
        , random=list(author=c(b0 ~1))
        #, random=list(author=c(b1 ~1))
        #, random=list(author=c(b0+b1 ~1))
        , start=.start   
        , weights=varPower(form = ~fitted(.)|author)
)
mm1
plot( stem~dbh, col=author, data=ds, xlab="dbh [cm]", ylab="")
mtext( "stem biomass [kg]", 2, 2.5, las=0)
yds <- predict(mm1, level=0) 
lines( yds ~ dbh, data=ds, col="black", lwd=2)

#authors <- levels(ds$author)
authors <- c("Heinsdorf","Cienciala","Duvigneaud2")
for( authorI in authors){
    dsNa <- ds; dsNa$author[] <- authorI
    try(lines( predict(mm1, level=1, newdata=dsNa) ~ dbh, data=dsNa, col=which(authorI == levels(ds$author)) ))
}
legend( "topleft", inset=c(0.01,0.01), legend=c(sort(authors),"Population"), col=c(which(levels(ds$author) %in% authors),"black"), lty=1, lwd=c(rep(1,length(authors)),2) )

#sd <- sqrt( varResidPower(gm1, pred=predict(mm1, level=0)) )
mm1a <- attachVarPrep( mm1, fVarResidual=varResidPower)     # calculate symbolic derivatives used in variance prediction
sd <-varPredictNlmeGnls(mm1a, newdata=ds)[,"sdInd"]     # variance components
lines( yds+sd ~ dbh, data=ds, col="grey" )
lines( yds-sd ~ dbh, data=ds, col="grey" )


## ----, spar=TRUE---------------------------------------------------------
apply( varPredictNlmeGnls(mm1a, newdata=ds), 2, median )


## ----predictGroup, spar=TRUE, cache=TRUE---------------------------------
yNew <- predict(mm1, newdata=dsNew, level=0 )
yAgg <- sum(yNew)
(varSumCompM <- varSumPredictNlmeGnls(mm1a, newdata=dsNew, retComponents = TRUE))
(cvAgg <- varSumCompM["sdPred"] / yAgg )     # 26% uncertainty


## ----bootstrap-----------------------------------------------------------
require(mnormt)     # multivariate normal
nBoot <- 400

beta <- rmnorm(nBoot, fixef(mm1), varFixef(mm1) )       # random realizations of fixed effects
b0 <- rnorm(nBoot, 0 , sqrt(varRanef(mm1)) )            # random realizations of b0
 # extract the coefficients of heteroscadistic varaicne model 
sigma0 <- mm1$sigma                                      
delta <- mean( coef(mm1$modelStruct$varStruct,uncons = FALSE, allCoef = TRUE), nBoot, replace=TRUE)
 # do the bootstrap
res <- sapply( 1:nBoot, function(i){
            yNewP <- (beta[i,"b0"]+b0[i]) * dsNew$dbh^(beta[i,"b1"])
            sigma <- sigma0 * yNewP^delta
            eps <- rnorm( nrow(dsNew), 0, sigma)                      # random realizations of resiudal base
            yNewT <- yNewP +eps
            yAggi <- sum(yNewT)
        })
sd(res)/yAgg        # confirms cv=26%


